%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function X = SimulateMarkovChain(P,T,nrep,x0)
% X = SimulateMarkovChain(P,T,nrep,x0)   
%   Simulate a Markov chain for T periods starting from x0;
%   
% Inputs:
% P     n x n     transition matrix with columns summing to one  
%                 P(i,j) is probability to go to i from j
% T     scalar    Number of periods in simulation
% nrep  scalar    number of repetitions, defaults to 1.
% x0    scalar    integer in [1,n] index of state in period 0
%     or 1 x nrep
%                 defaults to round(n/2)
%
% Outputs
%
% X     T x nrep integers in [1,n] indices of states in periods 1 to T

if ~exist('nrep','var')
    nrep = 1;
end

if ~exist('x0','var')
    x0 = round(n/2);
end

n = size(P,1);

assert(size(P,2) == n);
assert(all(abs(sum(P)-1)<1e-10));
assert(all(x0 >= 1) && all(x0 <= n) && all(floor(x0) == x0)); % floor(x0) == x0 checks that it is an integer
assert(nrep > 0 && floor(nrep) == nrep);

X = zeros(T+1,nrep);
X(1,:) = x0;

if nrep > 1
    for t = 2:T+1
        for i = 1:n
            I = X(t-1,:) == i;
            nI = nnz(I);
            if nI > 0
                X(t,I) = DrawDisc(nI,P(:,i));
            end
        end
    end
else
    for t = 2:T+1   
        X(t) = DrawDisc(1,P(:,X(t-1)));
    end
end

X = X(2:T+1,:);