%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

% Check if we are running inside a batch script:
% if so, we take experiment variable from outer script.
% otherwise, this is the main script and we set experiment here
close all; clc;
clear all;

CALIBRATE_BETA = false;

addpath('EDS');
addpath(genpath('share'));

if exist('parpool','file')
    poolobj = gcp('nocreate');
    if isempty(poolobj)
        parpool(6);
    end
end

global Params;

LOAD_SAVED = true; 
DO_SIM_MOMENTS = true;



Params.RepAgent = false;


%----------------------------------------------------------------------
%set parameters here
%----------------------------------------------------------------------
Params.betaHet = true;
Params.dox = true;
Params.doTails = true
%----------------------------------------------------------------------
%----------------------------------------------------------------------



initparams;
Params.EDSM = 80;

% only do accuracy checks for the baseline case
DO_ACCURACY =  Params.betaHet & Params.dox & Params.doTails;  



%% load saved parameters /  initialize

%start with linear (then automatically switch to non-linear)
order = 1;
classification = Params.States;
seed = 453265674;



load('data/GridAndPolicies_RepAgent.mat','Sgrid','PCGrid','polfunc');
% First guess of the forecasting rule is the law of motion for capital
% under complete markets
fr1 = AMpolyint_step3( polfunc,Sgrid);

if ~Params.dox
    Sgrid = Sgrid(:,1:end-1);
    PCGrid = PCGrid(:,1:end-1);
end
if ~Params.doTails
    Sgrid = Sgrid(:,1:6);
    PCGrid = PCGrid(:,1:6);
end    
Params.Sgrid = Sgrid;
Params.PCGrid = PCGrid;
Params.eps = 0.0001;
K = mean(Params.Sgrid(:,nameInd('k',Params.aggnames)))*0.95;



%Initial state vector for simulation (something close to steady state is
%good)
[~, us, ul, zeta, lambdas, Q] = CompleteMarketsSteadyState( Params  );
S0 = [0 Params.labor.m(1:2)' K invlogistic([us ul]) Q  0];  % use Q = 1 to coincide with initialization of D0
if ~Params.dox
    S0 = S0(1:end-1);
end
if ~Params.doTails
    S0 = S0(1:6);
end    




% This is the initial distribution over idiosyncratic states.
[ us, ul ] = LaborMarketSteadyState(  );
theta = linspace(-0.2,0.2,5);
disc = (1:Params.npp)';
p = [ 1-us-ul; us; ul];
if Params.betaHet
    p = [Params.fracLowBeta*p; (1-Params.fracLowBeta)*p];
end
assert( abs(sum(p)-1) < 1e-10);
D0 = zeros(Params.Sim.n,Params.npp);
for ip = 1:Params.npp
    I = disc == ip;
    D0(:,ip) = getprobs2D(repmat(K,1,5)/(1-Params.omega), exp(theta), p(ip)*ones(1,5)/5, Params.Sim.Grid1D_k', exp(Params.Sim.Grid1D_theta'));
end


%%  Initial guesses

polfunc  = AMpolyint_step1( Params.Sgrid, order, classification );

aofS = zeros(length(Params.Sgrid),Params.npp*Params.nkp);
for i = 1:length(Params.Sgrid)
    for j = 1:Params.npp
        ind = (j-1)*Params.nkp;
        aofS(i,ind+[1:Params.nkp]) = 1.055*Params.kpgrid;
    end
end

polfunc = AMpolyint_step2(polfunc,aofS);
forecastRule = AMpolyint_step1(Params.Sgrid,order, classification);
forecastRule = AMpolyint_step2(forecastRule,fr1);
clear fr1 aofS;


%% pre-compute if necessary
if ~Params.doTails
     [~, ~, ~, ~, ~, Params.Q] = CompleteMarketsSteadyState( Params  );
end


%% Load saved results
fname = ['data/GridAndPolicies' Params.fnameSuffix '.mat'];
disp(['Attempting to load ' fname])
if LOAD_SAVED && exist(fname,'file')
    load(fname,'Sgrid','PCGrid','polfunc','forecastRule','eps');
    Params.Sgrid = Sgrid;
    Params.PCGrid = PCGrid;
    Params.eps = eps;
    order = 1.5;
    forecastRule  = AMpolyint_step1( Sgrid, 1, Params.States, forecastRule );  % linear forecast rule
    ShortSim = false;
    disp('found it')
else
    % Simulate to burn-in cross-sectional distribution
    disp('Did not find it. Using ShortSim = true')
    
    ShortSim = true;
    
    burn = false;
    DoCY = false;
    DoTodaFarmer = false;

    rng(seed);
    AggEps = diag(Params.Sigma(1:Params.nz))*randn(100,Params.nz)';
    [simstates, D0] = SimulateDynamics(polfunc,S0,D0,AggEps,Params,burn,DoCY,DoTodaFarmer);
    clear AggEps;
    S0 = simstates(:,end)';

    DoTodaFarmer = true;
    AggEps = diag(Params.Sigma(1:Params.nz))*randn(500,Params.nz)';
    [simstates, D0] = SimulateDynamics(polfunc,S0,D0,AggEps,Params,burn,DoCY,DoTodaFarmer);
    clear AggEps;
    S0 = simstates(:,end)';

    %  what is the level of K in S0?
    assert( S0(nameInd('k',Params.aggnames)) > min(Params.Sgrid(:,nameInd('k',Params.aggnames))) && S0(nameInd('k',Params.aggnames)) < max(Params.Sgrid(:,nameInd('k',Params.aggnames)))  )

    
end

%% solve for the policy rules

[polfunc, forecastRule, Sgrid, PCGrid, simstates, finalDist, order] = Solve(polfunc, forecastRule, order, S0, D0, seed, ShortSim);
eps = Params.eps;
save(['data/GridAndPolicies' Params.fnameSuffix '.mat'], 'Sgrid', 'PCGrid', 'forecastRule', 'polfunc', 'eps');
clear eps PCGrid Sgrid;


%% Calibrate beta(s)

if CALIBRATE_BETA
    targets = [3.32*4; 1-0.834];
    par0 = [Params.beta(1); Params.beta(end)];
    if ~Params.betaHet
        targets = targets(1);
        par0 = par0(1);
    end
    Params.polfunc_start = polfunc;
    Params.forecastRule_start = forecastRule;
    calibrate_moments(invlogistic(par0),targets, order,S0,D0,seed);
end


%% Simulated moments

if DO_SIM_MOMENTS
    rng(76342);
    Tsim = 2000;
    AggEps = diag(Params.Sigma(1:Params.nz))*randn(Tsim+Params.burn,Params.nz)';  
    simstates = SimulateDynamics(polfunc,S0,D0,AggEps,Params); 
    save(['data/moments' Params.fnameSuffix '.mat'], 'simstates');
    [y, c, u,~,~,~,~,R] = simulated_static(simstates);
    dc = diff(log(c));
    dy = diff(log(y));
    du = diff(u);
    
    
    X = [log(y(2:end))' log(c(2:end))' u(2:end)' dy' dc' du'];
    std_ycu = std(X)
    cov_ycu = corrcoef(X)
    
    std_R = std(R');
    corr_YR = corrcoef(y', R');
    
    save(['data/moments' Params.fnameSuffix '.mat'], 'std_ycu', 'cov_ycu', 'simstates', 'std_R', 'corr_YR');
    
end

%% Find Risky Steady State
disp('Finding risky steady state')
AggEps = zeros(Params.nz,Params.Tsim+Params.burn);
[simstates, finalDist] = SimulateDynamics(polfunc,S0,D0,AggEps,Params);
clear AggEps;
S0 = simstates(:,end)';

% check capital output ratio
[~, ~, ~, z, L, k] = simulated_static(repmat(S0',1,2));
KY = exp(- z) .* (k./L).^(1-Params.alpha);
disp(['RSS K-Y ratio: ' num2str(KY)])



%% Simulate recessions

GreatRecession(S0,finalDist,polfunc,2007);
GreatRecession(S0,finalDist,polfunc,1977);

%% Analyze the distribution of wealth and MPC


disp('Wealth')
disp('Computing Earnings Gini')
tic
[giniIndex, Lorenz] = gini(sum(finalDist,2), Params.Sim.k.*exp((1-Params.by)*Params.Sim.theta), false);
toc
DistributionDisplay(giniIndex,Lorenz);

disp('Earnings')
tmpA = exp((1-Params.by)*Params.Sim.theta);
tmpA = kron(Params.employed_n, tmpA);
tmpB = finalDist(:);
tmpAA = [];
tmpBB = [];
for i = 1:length(Params.transgrid)
    tmpAA = [tmpAA; exp(Params.transgrid(i)) * tmpA];
    tmpBB = [tmpBB; Params.transprob(i)*tmpB];
end
assert( abs(sum(tmpBB) - 1.0) < 1e-5)
disp('Computing Earnings Gini')
tic
[giniIndex, Lorenz] = gini(tmpB, tmpA, false);
toc
DistributionDisplay(giniIndex,Lorenz);


if exist('~/MyLib/compecon')
    mpc = get_mpc(S0,finalDist, polfunc);
    disp(['Average MPC = ' num2str(mpc)])
end

%% Accuracy checks

if DO_ACCURACY
    
    HCheck( S0, finalDist, polfunc, forecastRule)
    EulerErrorNL_script;
    
end




%% WRAP UP
disp('done')

