%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 


close all;
clear all;
clc

DO_CALIBRATE = false;

addpath('EDS');
addpath(genpath('share'));


if exist('parpool','file')
    poolobj = gcp('nocreate');
    if isempty(poolobj)
        parpool(6);
    end
end

global Params;


%set the baseline parameters
Params.RepAgent = true;
Params.betaHet = false;
Params.dox = true; 
Params.doTails = true;

initparams;

Params.EDSM = 80;


seed = 453265674;



%%  Initial guess

%Initial conditions are important.  


[K, us, ul, zeta, lambdas, Q] = CompleteMarketsSteadyState( Params );


%Initial state vector for simulation (something close to steady state is
%good)
if Params.dox
    S0 = [0 Params.labor.m(1:2)' K invlogistic([us ul]) Q 0 ];
else
    S0 = [0 Params.labor.m(1:2)' K invlogistic([us ul]) Q   ];
end


%initial guess policy rule
polfunc.d = 10;
polfunc.coef = [ K    0.1 0 0 0.8 0 0  0 0 0 0]';
polfunc.coef(1) = polfunc.coef(1) - dot(polfunc.coef(2:end), S0(Params.States));
polfunc.ord = 1;
polfunc.cls = polfunc.ord * Params.States;
polfunc.clsmask = Params.States;





%% Simulate for initial grid
rng(seed);
AggEps = diag(Params.Sigma(1:Params.nz))*randn(Params.Tsim+Params.burn,Params.nz)';
simstates = SimulateDynamicsRA(polfunc,S0,AggEps,Params);
clear AggEps;

[Params.Sgrid, ~, Params.PCGrid] = EDS_get_grid(simstates,0.2);
Params.eps = 0;


% This guess has a higher beta and a higher mean capital stock than the
% deterministic steady state, but it is close to the risky steady state.
load('LinearGuess/CompMkts_results.mat','oo_');
Sgrid = oo_.endo_simul(1:6,:);
Sgrid = EDS_get_grid(Sgrid,0.2);
n = min(size(Params.Sgrid,1),size(Sgrid,1));
Params.Sgrid = Params.Sgrid(1:n,:);
Sgrid = Sgrid(1:n,:);
Params.Sgrid(:,1:6) = Sgrid;
clear Sgrid oo_;
Params.eps = 0;




%% Solve first with a linear policy approximation

% approximation parameters
order = 1;
classification = Params.States*1;
polfunc = AMpolyint_step1(Params.Sgrid,order,classification,polfunc);


[polfunc, Sgrid, PCGrid, simstates] = SolveRepAgent(polfunc, order, seed);


save(['data/GridAndPolicies' Params.fnameSuffix '.mat'], 'Sgrid', 'polfunc', 'PCGrid');
clear Sgrid PCGrid;


%% Solve second with a mixed policy approximation
disp('-------Switching to order 1.5----------')
% approximation parameters
order = 1.5;
classification(1:6) = 2;
polfunc = AMpolyint_step1(Params.Sgrid,order,classification,polfunc);


[polfunc, Sgrid, PCGrid, simstates] = SolveRepAgent(polfunc, order, seed);


save(['data/GridAndPolicies' Params.fnameSuffix '.mat'], 'Sgrid', 'polfunc', 'PCGrid');
clear Sgrid PCGrid;


%% Simulated moments
%Simulate
rng(76342);
AggEps = diag(Params.Sigma)*randn(Params.Tsim+Params.burn,4)';
S0 = Params.Sgrid(30,:);
simstates = SimulateDynamicsRA(polfunc,S0,AggEps,Params);
clear AggEps;
[y, c, u, ~,~,~,~,R] = simulated_static(simstates);


dc = diff(log(c));
dy = diff(log(y));
du = diff(u);

X = [log(y(2:end))' log(c(2:end))' u(2:end)' dy' dc' du'];
std_ycu = std(X)
cov_ycu = corrcoef(X)

save(['data/moments' Params.fnameSuffix '.mat'], 'std_ycu', 'cov_ycu','simstates');


%% Find Risky Steady State
disp('Finding risky steady state')
AggEps = zeros(Params.nz,Params.Tsim+Params.burn);
S0 = Params.Sgrid(1,:);
simstates = SimulateDynamicsRA(polfunc,S0,AggEps,Params);
clear AggEps;
S0 = simstates(:,end)';


%% Great Recession

GreatRecession(S0,[],polfunc,2007);
GreatRecession(S0,[],polfunc,1977);

%% Calibrate beta
if DO_CALIBRATE
   calibrate 
end
