%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [ epsgrid, transgrid, probs, nquad  ] = ShockGrid( x,zetap, lambdap, us,ul, Params, exponentiate )
% Returns quadrature grid and weights over idiosyncratic shocks (permanent
% and transitory)
%
% Inputs:
% x         scalar      risk factor in t
% zetap     scalar      job-separation rate in t+1
% lambdap   scalar      job-finding rate in t+1
% us        scalar      short-term unemployent rate in t
% ul        scalar      long-term unemployent rate in t
% Params    structure   Parameter structure 
% exponentiate logical  if true return shock x transformed by exp((1-Params.by) * x)
%
% Outputs:
% epsgrid   nquad x 3   Array of quadrature points for each of three mixture components
% transgrid nquad x 1   Array of quadrature points for transitory shock
% probs     nquad x 3   Associated probabilities
% nquad     scalar      size of grid in first dimension
%
% Explanation:
% epsgrid and transgrid are a product grid of the two shocks and probs are
% the product of the two shock probabilities.





MUp  = get_MU( x,zetap, lambdap, us,ul, Params );

epsgrid = [MUp(1)+Params.epsgrid1  MUp(2)+Params.epsgrid2  MUp(3)+Params.epsgrid3];
transgrid = Params.transgrid;

if exponentiate
    epsgrid = exp((1-Params.by)*epsgrid);
    transgrid = exp((1-Params.by)* transgrid);
end

epsprob = [ Params.epsprob1  Params.epsprob2  Params.epsprob3];

epsgrid = kron(epsgrid,ones(Params.ntrans,1));
epsprob = kron(epsprob,ones(Params.ntrans,1));


transgrid = kron(ones(Params.neps,1),transgrid);
transitoryprob = kron(ones(Params.neps,1),Params.transprob);


probs = bsxfun(@times,transitoryprob, epsprob);


nquad = Params.neps * Params.ntrans;



end

