%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [Kp, finalDist, CYPcts] = SimulateCore(polfunc,S,Dist0,Params,xold,DoCY)
% Simulates for one time period
% [Kp, finalDist] = SimulateCore(polfunc,AggState,Dist0,Params, xold)
%
% Inputs:
% polfunc      1 x npp*nkp   policy rule vector
%          OR  structure     in this case it is interpolated from (S0,K)
% S       1 x nstates        this is the aggregate state entering this period including aggregate shocks from this period
% Dist0    structure         distribution entering the period (before idio shocks occur)
% Params   structure
% xold     float             value of x in previous period that determines the distribution of shocks realized today.
% DoCY scalar           Logical -- return distributional statistics of consumption and income
%
% Outputs:
% Kp        scalar           aggregate savings (at end of period)
% finalDist structure        distribution at the start of next period (before next period's shocks)
% CYPcts    5 x 4            matrix of percentiles. rows are percentiles
%                            (20, 50, 80, 90, 95). columns are consumption, disposable income, earnings, total income
%

if ~exist('DoCY','var')
    DoCY = false;
end


iuhat = 2;
ik = 3;


n = Params.nsim;


theta = Dist0.theta;
k = Dist0.k; %k is relative to e^(theta*(1-by))
disc = Dist0.disc;

wht = ones(n,1);
Heps = zeros(n,1);

nd =round(Params.omega*n); %number dead


savings = S(ik);

nkp  = Params.nkp;
npp = Params.npp;




[w, R, IPES,  lambda, zeta] = static(S, Params);
u = logistic(S(iuhat));
MU = get_MU( xold, Params );  % FIXME: update here


%mortality--make sure exactly right number die each period
dead = false(n,1);
tmp1 = ceil(n*rand(10*nd,1));
for ndeadind = nd:10*nd
    tmp = unique(tmp1(1:ndeadind));
    if length(tmp) == nd, break; end
end
dead(tmp) = true;
assert(sum(dead) == nd);
oldtheta = theta;
theta(dead) = 0;


%shock to discrete state
r = repmat(rand(size(disc)),1,Params.npp);
Tr = transProb(disc,1:Params.npp,lambda,zeta,Params.betaTrans);
Tr = cumsum(Tr,2);
Tr = r > Tr;
disc = sum(Tr,2) + 1;





%post-stratify on employment
I = Params.employed(disc);
wht(I)  = (1-u)/mean(I);
I = ~I;
wht(I)  = u/mean(I);


%generate shocks to theta
r = rand(n,1);
s2 = r < Params.p2;
s3 = ~s2 & r< (Params.p2+Params.p3);
s1 = ~s2 & ~s3;

ns1 = nnz(s1);
ns2 = nnz(s2);
ns3 = nnz(s3);

Heps(s1) = MU(1) + Params.sigma1*randn(ns1,1);
Heps(s2) = MU(2) + Params.sigma2*randn(ns2,1);
Heps(s3) = MU(3) + Params.sigma3*randn(ns3,1);


%impose minimum theta
Heps = max(Heps,-12-theta);

Heps(dead) = 0;

%rescale so mean e^theta = 1 in both employed and unemployed
for ie = 0:1
    I = Params.employed(disc) == ie;
    xs = 1/mean(exp(theta(I)+Heps(I)));
    Heps(I) = log(xs) + Heps(I);
    theta(I) = theta(I) + Heps(I);
    
end


%_-----_



k(dead) = 0;
K = mean(wht.*exp(oldtheta*(1-Params.by)).*k);
AnnuityAdjustment = savings/K;
k = k*AnnuityAdjustment;
clear K;



%adjust k from last period's normalization to this period's
k = k./exp(Heps*(1-Params.by));


%generate the transitory shock
trans = exp(-Params.sigmae^2/2 + Params.sigmae*randn(n,1));

if isstruct(polfunc)
    agridUE = AMpolyint_step3(polfunc,S);
else
    agridUE = polfunc;
end


assert(all(abs(imag(agridUE)) < 1e-10))

if DoCY
    cindiv = zeros(size(k));
    yindiv = zeros(size(k,1),3);
end

for ip = 1:npp
    I = disc == ip;
    [coh, income] = get_coh(k(I),R,IPES,ip,trans(I),Params);
    
    
    assert(all(abs(imag(coh)) < 1e-10))
    
    ind = (ip-1)*nkp;
    agrid = agridUE(ind+(1:nkp));
    
    k(I) = interpagridkpgrid(agrid,Params.kpgrid,coh);
    
    if DoCY
        cindiv(I) = coh - k(I);
        yindiv(I,:) = income  ;
    end
end


savings = (1-Params.omega)*mean(wht.*exp(theta*(1-Params.by)).*k);
Kp = savings;

if DoCY
    
    cindiv =  exp(theta*(1-Params.by)) .* cindiv;
    
    %y: first column is disposable income, second is earnings, third is
    %total income
    yindiv(:,[1 3]) =  repmat(exp(theta*(1-Params.by)),1,2) .* yindiv(:,[1 3]);
    yindiv(:,2) =  w.*(exp(theta*(1-Params.by)) .* yindiv(:,2)/IPES).^(1/(1-Params.by));
    yindiv(:,1) = yindiv(:,1) + yindiv(:,3);
    yindiv(:,3) = yindiv(:,2) + yindiv(:,3);
    
    CYPcts = MyPercentiles( [cindiv yindiv],[0.2;0.5; 0.8; 0.9; 0.95] );
    
end 

%-----


finalDist.k = k;
finalDist.disc = disc;
finalDist.theta = theta;
finalDist.wht = wht;


%close(wb)
end
