%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [Kp, Dist1, CYPcts] = SimulateCoreNonStoch(polfunc,S,Slag,Dist0,Params,DoCY,DoTodaFarmer)
% Simulates for one time period using non-stochastic simulation
% [Kp, Dist1] = SimulateCoreNonStoch(polfunc,AggState,Dist0,Params)
%
% Inputs:
% polfunc      1 x npp*nkp   policy rule vector
%          OR  structure     in this case it is interpolated from (S0,K)
% S       1 x nstates        this is the aggregate state entering this period including aggregate shocks from this period
% Slag     float             value of x in previous period that determines the distribution of shocks realized today.
% Dist0    nk * ntheta, 6     array of probabilities on grid.  distribution at the start of this period (before this period's shocks)
% Params   structure
% DoCY scalar           Logical -- return distributional statistics of consumption and income
%
% Outputs:
% Kp        scalar           aggregate savings (at end of period)
% Dist1    nk * ntheta, 6     distribution at the start of next period (before next period's shocks)
% CYPcts    5 x 2            matrix of percentiles. rows are percentiles
%                            (20, 50, 80, 90, 95). columns are consumption, disposable income
%


iQ = 7:12;
Qxi = exp(-0.5*(1-Params.by)*Params.sigmae^2+ 0.5*(1-Params.by)^2*Params.sigmae^2);

ntheta = numel(Params.Sim.Grid1D_theta);
nk = numel(Params.Sim.Grid1D_k);

if ~exist('DoCY','var')
    DoCY = false;
end

if ~exist('DoTodaFarmer','var')
    DoTodaFarmer = false;
end



if Params.dox
    xlag = Slag( nameInd('x',Params.aggnames)  );
else
    xlag = 0;
end
iushat = nameInd('ushat',Params.aggnames);
iulhat = nameInd('ulhat',Params.aggnames);
uslag = logistic( Slag( iushat ) );
ullag = logistic( Slag( iulhat ) );

[~, R, IPES,  lambda, zeta] = static(S, Params);

% Test the marginal over discrete states in the incoming distribution
% Remove this later.
% disctest = [1-uslag-ullag uslag ullag];
% if Params.betaHet
%     disctest = [Params.fracLowBeta*disctest  (1-Params.fracLowBeta)*disctest];
% end
% assert( all( abs( sum(Dist0) - disctest) < 1e-6));


Dist1 = zeros(Params.Sim.n,Params.npp);
Kp = zeros(Params.npp,1);

% These arrays are only used if DoCY
CGrid = [linspace(0,20,1200) linspace(21,100,80)];
YGrid = CGrid;
DistC = zeros(numel(CGrid),Params.npp);
DistY = zeros(numel(YGrid),Params.npp);




[ epsgrid, transgrid, shockprobs, nquad  ] = ShockGrid( xlag,zeta, lambda, uslag,ullag,  Params , false );

agridUE = AMpolyint_step3(polfunc,S);


%parfor here
parfor jp = 1:Params.npp
    
    ind = (jp-1)*Params.nkp;     %#ok<*PFBNS>
    agrid = agridUE(ind+(1:Params.nkp));
    
    for ip = 1:Params.npp
        P = Dist0(:,ip);
        
        for component = 1:3
            pp = transProb(ip,jp,lambda,lambda*Params.labor.LambdaEllFactor,zeta,Params.betaTrans)...
                 * ComponentProbs(ip,jp,component,Params.p2,Params.p3);
            
            if pp > 0
                
                Pij = pp * repmat(P,nquad,1) .* kron(shockprobs(:,component),ones(size(P,1),1));
                Heps = kron(epsgrid(:,component),ones(size(P,1),1));
                
                %update theta, adjust k to be relative to e^(theta*(1-by)) and mortality
                theta = min( ...
                        max(   [repmat(Params.Sim.theta,nquad,1)+Heps; zeros(Params.ntrans,1)],...
                                Params.Sim.Grid1D_theta(1) ),...
                                Params.Sim.Grid1D_theta(end) );
                k = min( ...
                    max(   [repmat(Params.Sim.k,nquad,1)./exp(Heps*(1-Params.by)); zeros(Params.ntrans,1)], ... %k is relative to e^(theta*(1-by))
                            agrid(1) ),...
                            agrid(end) );
                        
                Pij = [(1-Params.omega)*Pij; Params.omega*sum(Pij)*Params.transprob];
                
                %transitory income and cash on hand relative to e^(theta*(1-by))
                income = Params.employed_n(jp) * IPES  ...
                        * [kron(exp((1-Params.by)*transgrid),ones(size(P,1),1)); exp((1-Params.by)*Params.transgrid)];
                coh = income + R * k;
                            
                kp = min( ...
                     max( interpagridkpgrid(agrid,Params.kpgrid,coh), ...
                     Params.Sim.Grid1D_k(1) ), ...
                     Params.Sim.Grid1D_k(end) );
     
                Kp(jp) = Kp(jp) + (1-Params.omega) * dot(kp .* exp(theta*(1-Params.by)) ,Pij);
                
                Dist1(:,jp) = Dist1(:,jp) + getprobs2D(kp',exp(theta'),Pij',Params.Sim.Grid1D_k',exp(Params.Sim.Grid1D_theta'));
                
                if DoCY
                    c = min( (coh - kp) .* exp(theta*(1-Params.by)), CGrid(end)) ;
                    dispInc = min( (income + (R-1) * k) .* exp(theta*(1-Params.by)) , YGrid(end) );
                    DistC(:,jp) = DistC(:,jp) + getprobs1D(c',Pij',CGrid)';
                    DistY(:,jp) = DistY(:,jp) + getprobs1D(dispInc',Pij',YGrid)';
                end
                
            end
        end
    end
end

% compute marginal distribution over theta
MarginalTheta = zeros(ntheta,Params.npp);
for i = 1:Params.npp
    MarginalTheta(:,i) = sum(reshape(Dist1(:,i),ntheta,nk),2);
end


% Adjust the probabilities on the final grid point to hit the target Q
% listed in S
if Params.doTails && DoTodaFarmer
    iQind = [1 2 3 1 2 3];
    iQbyind = [4 5 6 4 5 6];
    
    for i = 1:Params.npp
        Ptotal = sum(MarginalTheta(:,i));
        Q1targ = S(iQ(iQind(i)));
        Q1bytarg = S(iQ(iQbyind(i))) / Qxi;
         
        p = discreteApproximation(Params.Sim.Grid1D_theta',@DistMoments,[Q1targ; Q1bytarg],MarginalTheta(:,i)'/Ptotal, zeros(2,1));
        
        scale = p'./(MarginalTheta(:,i)/Ptotal);
        scale(MarginalTheta(:,i)==0) = 0;
        tmp = bsxfun(@times,reshape(Dist1(:,i),ntheta,nk) , scale);
        assert( abs(sum(tmp(:)) - Ptotal) < 1e-7);
        Dist1(:,i) = tmp(:);
        
%         assert( abs( dot(Dist1(:,i),exp(Params.Sim.theta))/sum(Dist1(:,i)) - Q1targ) < 1e-6)
%         assert( abs( dot(Dist1(:,i),exp(Params.Sim.theta*(1-Params.by)))/sum(Dist1(:,i)) - Q1bytarg) < 1e-6)
%         
    end

end

% aggregate capital is sum over groups
Kp = sum(Kp);



if DoCY
    % collapse the distribution over jp
    % I did it this way to allow for parallelization (i.e. map reduce)
    DistC = sum(DistC,2);
    DistY = sum(DistY,2);
    
    CGrid = CGrid(DistC > 1e-7);
    DistC = DistC(DistC > 1e-7);
    DistC = DistC / sum(DistC);
    
    YGrid = YGrid(DistY > 1e-7);
    DistY = DistY(DistY > 1e-7);
    DistY = DistY / sum(DistY);
    
    % -- find percentiles --
    DistC = cumsum(DistC);
    DistY = cumsum(DistY);
    
    assert( abs(DistC(end) - 1) < 1e-10) 
    assert( abs(DistY(end) - 1) < 1e-10) 
    
    CYPcts(:,1) = interp1(DistC,CGrid,[.20; .50; .80; .90; .95],'linear');
    CYPcts(:,2) = interp1(DistY,YGrid,[.20; .50; .80; .90; .95],'linear');

end


% test the marginal over discrete states
% us = logistic( S( iushat ) );
% ul = logistic( S( iulhat ) );
% disctest = [1-us-ul us ul];
% if Params.betaHet
%     disctest = [Params.fracLowBeta*disctest  (1-Params.fracLowBeta)*disctest];
% end
% assert( all( abs( sum(Dist1) - disctest) < 1e-6));


end