%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [polfunc, forecastRule, Sgrid, PCGrid, simstates, finalDist, order] = Solve(polfunc, forecastRule, order, S0, D0, seed, ShortSim)
% ShortSim is a logical (default = false).  If it is true, then we do a few
% short simulations instead of one long one.  This increases the dependence
% on the initial condition, but it is more robust because we don't end up
% too far from where we start and leave the grid on which we solve the
% model.  I use ShortSim in the initial runs when looking for the
% rough location of the grid and then don't use it for the production
% version of the code.  In effect it is a useful way of generating initial
% conditions.

global Params;

if ~exist('ShortSim','var')
    ShortSim = false;
end

if ShortSim
    disp('Solving with ShortSim = true')
end


ik = nameInd('k',Params.aggnames);
maxit = [30 5000];

for it = 1:maxit(1)
    disp(['iteration ' num2str(it)])
    
    
    
    
    disp('updating polfunc')
    for i = 1:maxit(2)
        polfunc2 = updatepolfunc(polfunc,forecastRule,Params);  
        
        tmp = (AMpolyint_step3(polfunc2,Params.Sgrid) - AMpolyint_step3(polfunc,Params.Sgrid))./(AMpolyint_step3(polfunc2,Params.Sgrid) + 1);
        tst = max(abs(tmp(:)));
        
        if mod(i,20) == 0
            tst
        end
        
        polfunc.coef = polfunc2.coef;
        
        
        if tst < 1e-5, break; end
    end
    
    %simulate a panel of households and record aggregate states
    disp('Simulating')
    rng(seed);
    

    if ShortSim
        % small simulation for gradual grid convergence
        AggEps = diag(Params.Sigma(1:Params.nz))*randn(300,Params.nz)';  
        [simstates, finalDist] = SimulateDynamics(polfunc,S0,D0,AggEps,Params,0); 
        rng(12438)
        AggEps = diag(Params.Sigma(1:Params.nz))*randn(300,Params.nz)';  
        [simstates1, finalDist] = SimulateDynamics(polfunc,S0,D0,AggEps,Params,0); 
        simstates = [simstates simstates1];
        rng(890723)
        AggEps = diag(Params.Sigma(1:Params.nz))*randn(300,Params.nz)';  
        [simstates1, finalDist] = SimulateDynamics(polfunc,S0,D0,AggEps,Params,0); 
        simstates = [simstates simstates1]; 
    else
        AggEps = diag(Params.Sigma(1:Params.nz))*randn(Params.Tsim+Params.burn,Params.nz)';  
        [simstates, finalDist] = SimulateDynamics(polfunc,S0,D0,AggEps,Params); 
    end
    
    
    %update the grid
    [newGrid, neweps, newPCGrid] = EDS_get_grid(simstates,0.2);
    
    
    %check convergence
    gridconverged = true;
    for i=1:size(newGrid,1)
        mindist = min(EDS_dist(Params.PCGrid,newPCGrid(i,:)));
        if mindist > (neweps+Params.eps)
            [mindist  (neweps+Params.eps)]
            gridconverged = false;
            break
        end
    end
    
    if any(abs(mean(newGrid) - mean(Params.Sgrid))./std(Params.Sgrid) > 0.5)
        disp('means not converged')
        gridconverged = false;
        
%         if it <= 3 &&  mean(newGrid(:,ik)) < mean(Params.Sgrid(:,ik))
%             newGrid = Params.Sgrid;
%             newGrid(:,ik) = Params.Sgrid(:,ik) * 0.8;
%         end
%         
%         if it <= 3 &&  mean(newGrid(:,4)) > mean(Params.Sgrid(:,4))
%             newGrid = Params.Sgrid;
%             newGrid(:,4) = Params.Sgrid(:,4) * 1.2;
%         end
    end
    
    if ~gridconverged
        disp('updating grid')
        Params.Sgrid = newGrid;
        Params.eps = neweps;
        Params.PCGrid = newPCGrid;
        polfunc = AMpolyint_step1( Params.Sgrid, order,polfunc.cls, polfunc );
        forecastRule = AMpolyint_step1( Params.Sgrid, order,forecastRule.cls, forecastRule );
    end
    
    
    if gridconverged
        
        disp('updating the forecasting rule')
        % KS regression
        try
            Kp = simstates(ik,2:end)';
            basisfcns = AMpolyint_basis(simstates(forecastRule.clsmask,1:end-1)', 1, forecastRule.cls(forecastRule.clsmask));


            HcoeffsNL2 = (basisfcns'*basisfcns + 0.00001 * eye(size(basisfcns,2)))\(basisfcns'*Kp);
            disp(['R^2 = ' num2str( 1-sum((Kp-basisfcns*HcoeffsNL2).^2)/sum((Kp-mean(Kp)).^2),12)])

            KS_test = (AMpolyint_step3(forecastRule,Params.Sgrid) - AMpolyint_basis(Params.Sgrid,1,forecastRule.cls(forecastRule.clsmask)) * HcoeffsNL2)./AMpolyint_step3(forecastRule,Params.Sgrid);
            KS_test = max(abs(KS_test(:)))

            forecastRule.coef = HcoeffsNL2;
        catch exc
            save tmp.mat
            rethrow(exc)
        end
        
        if order ==1
            disp('Swithching to order 1.5')
            order = 1.5;
            classification = Params.States*2;
            classification(7:12) = min(classification(7:12),1);  % sets 7:12 to 1 if they are not zero already
            polfunc = AMpolyint_step1( Params.Sgrid, order, classification, polfunc );
            %forecastRule = AMpolyint_step1( Params.Sgrid, order, forecastRule.cls, forecastRule );
        end
        
        KSconverged = KS_test < 1e-5
    end
    
    
    
    if gridconverged && KSconverged, break; end
    
end


% save results
Sgrid = Params.Sgrid;
PCGrid = Params.PCGrid;



end

