%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function CRHS = eulerresNL(polfunc,Kp,Params, Statei,savings)
% CRHS is n x npp array of consumption at t implied by RHS of Euler equation spanning t and t+1.
%
% Kp is aggregate capital at t+1 chosen in t
%
% Statei is aggregate state at t
%
% savings is an n x npp array of savings of individuals chosen t (these are
% the savings levels corresponding to the points at which we assess the
% Euler equation error.


nkp = size(savings,1);
npp = Params.npp;

ix = nameInd('x',Params.aggnames);
iushat = nameInd('ushat',Params.aggnames);
iulhat = nameInd('ulhat',Params.aggnames);





n = repmat(7,1,4);
mu = zeros(1,4);
SS= Params.Sigma;
var = diag(SS)^2;
[quadEps, quadW] = qnwnorm(n,mu,var);

  
% unpack
x = Statei(ix);
us = logistic(Statei(iushat));
ul = logistic(Statei(iulhat));

    
% initialize
MUexp = zeros(nkp,npp);
kp = zeros(Params.neps * Params.ntrans * nkp,1);   
CRHS = zeros(nkp,npp);

% loop over aggregate shocks
for epsi = 1:size(quadEps,1)
        
    % future aggregate state
    Sp =  StateUpdate(Kp,Statei, quadEps(epsi,:)',Params);
        
        
    % future prices amd labor transition probabilities
    [~, Rp, IPESp, lambdap, zetap] = static(Sp, Params);
        
    % get future policy rule
    agridUE = AMpolyint_step3(polfunc,Sp);  % 1 x 900,  lists the coh that leads to kpgrid
        
    % shocks depend on labor transitions
    [ epsgrid, transgrid, probs, nquad  ] = ShockGrid( x,zetap, lambdap, us,ul,Params ,true);  %this gives shocks raised to the power (1-by)
        
    for ip = 1:npp % loop over current income state    
        
        %MUnext = zeros( nkp,npp,3);  %last dimension corresponds to income mixture components
        for jp = 1:npp %loop over future period income state
            
            
            ind = (jp-1)*Params.nkp;
            agrid = agridUE(ind+ (1:Params.nkp));
            
            income = IPESp * Params.employed_n(jp) * transgrid;
            
            for component = 1:3
                pp = ComponentProbs( ip, jp, component , Params.p2, Params.p3) * ...
                     transProb(ip,jp,lambdap,lambdap*Params.labor.LambdaEllFactor,zetap,Params.betaTrans);
                
                if pp > 0
                    coh = repmat(income,1,nkp) +  repmat(Rp*savings(:,ip)',nquad,1)./repmat(epsgrid(:,component),1,nkp);  % this is a nquad x nkp matrix of coh/e^((1-by)theta')
                    kp(:) = interpagridkpgrid(agrid,Params.kpgrid,coh(:));  % for each coh, find how much we save
                end

                c = coh(:)- kp;  % consumption
                
                if any(c <=0)
                    save EE_assertion.mat;
                    error('Zero or negative consumption in eulerresNL')
                end
                
                %expected marginal utility times e^{-gamma (1-by) eps'}.  the
                %expectation is taken over idiosyncratic skill shocks
                %conditional on component
                MUexp(:,ip) = MUexp(:,ip)+ quadW(epsi) * pp * Rp.*sum(repmat(probs(:,component),1,nkp).*reshape((c.*repmat(epsgrid(:,component),nkp,1)).^(-Params.CRRA),nquad,nkp))';
            end
        end
    end
        
         
        
end  %end loop over aggregate shocks
    
    
CRHS = bsxfun(@times,MUexp,Params.beta).^(-1/Params.CRRA);
    

end