%  Replication files for Time-Varying Idiosyncratic Risk and Aggregate Consumption Dynamics 
%  Alisdair McKay  
%  May 1, 2017 
%  -------------------------- 

function [probs, Xg, Yg] = getprobs2D(x,y,p,X,Y)
% Takes a set of points (x,y) with probabilities p and returns a new set of
% probabilities on the grid given by the product of the vectors X and Y.
% All inputs should be row vectors.
%
% Ouputs (let m = length(X) and n = length(Y))
% all outputs are n x m arrays 
% Xg, Yg are simply the output from meshgrid(X,Y)
% probs are the associated probabilities of the points in Xg, Yg
%
% Only works with matlab version R2014b or later.


m = length(X); 
n = length(Y);
k = size(x,2);


% check the inputs
if any(diff(X)<=0) || any(diff(Y) <=0)
    error('X and Y must be strictly increasing grids');
end

if min(x) < X(1) || max(x) > X(end) || min(y) < Y(1) || max(y) > Y(end)
    error('x and y must lie in X and Y, respectively');
end

if size(x,1) > 1 || size(y,1) > 1 || size(p,1) > 1 || size(X,1) > 1 || size(Y,1) > 1
    error('All inputs must be scalar or row vector')
end

if size(y,2)~= k || size(p,2) ~= k
    error('x, y, and p must be the same size')
end

if any(p > 1) || any(p <0)
    error('Input p is not a valid probability distribution (an element is less than zero or greater than one).')
end

% find indices for each x and y

[Xg,Yg] = meshgrid(X,Y);

% these lines only work with Matlab R2014b or later
indx = interp1(X,1:m,x,'previous')+1; %use previous + 1 not next because of treatment when x is exactly equal to an element of X
indy = interp1(Y,1:n,y,'previous')+1;

% if you are using old Matlab and you have the CompEcon tool box then you
% can use these lines instead
%indx = lookup(X,x)+1;
%indy = lookup(Y,y)+1;

% the line above has an issue if the index is right on the last grid point
% so we correct it here
indx = min(indx,m);
indy = min(indy,n);

assert(all([indx indy]>1))





% scale to unit square
% If you can an error here about indexes not being integers, it is because
% you are using an old version of Matlab that doesn't support interp1 with
% method 'previous'
xscaled = (x-X(indx-1))./(X(indx) - X(indx-1));
yscaled = (y-Y(indy-1))./(Y(indy) - Y(indy-1));

% allocate on unit square
probs = getprobs2Dunitsquare(xscaled,yscaled);
assert(all(probs(:) >= 0))
assert(all(probs(:) <= 1))
assert(all(abs(sum(probs,1) - 1)<1e-6) )

% accumulate the results
subs = [indy'-1 indx'-1; indy'-1 indx'; indy' indx'-1 ;  indy' indx'];
vals = ((ones(4,1) * p ).*probs)';
vals = vals(:);

probs = accumarray(subs,vals,[n,m]);
probs = probs(:);

end
function probs = getprobs2Dunitsquare(x,y)
% takes a point in the unit square x,y
% returns a 4 x 1 array with the weights on (0,0), (1,0), (0,1), (1,1)
% it can be vectorized if x and y are row vectors and then probs is 4 x n
% this function works by imposing the following constraints
% mean on first dimension is x, mean on second dimension is x, the
% probabilites sum to 1, the expectation of the two dimensions is x*y
probs = [0 1 0 1; 0 0 1 1; 1 1 1 1; 0 0 0 1]\[x; y; ones(size(x)); x.*y];
end